#!/usr/bin/perl
#
# this software is released under the GNU Public License (http://www.gnu.org/)
# it is written by Eric Lewandowski <sigsegv@cox.net> and <tmancill@debian.org> 
#
# check_netsaint_statd.pl <host> [port]
#
# NetSaint plugin to check and see if a client is running netsaint_statd.
#
# modification history:
# 20011003/tm	plugin timeout now generates UNKNOWN instead of CRITICAL
##########################################################################

require 5.003;
BEGIN { $ENV{PATH} = '/bin' }
use Socket;
use POSIX;

$VERSION = "2.15";

sub usage;

my $TIMEOUT = 15;

my %ERRORS = ('UNKNOWN', '-1',
		'OK', '0',
		'WARNING', '1',
		'CRITICAL', '2');
my $remote = shift || &usage(%ERRORS);
my $port = shift || 1040;

my $remoteaddr = inet_aton("$remote");
my $paddr = sockaddr_in($port, $remoteaddr) || die "Can't create info for connection: #!\n";;
my $proto = getprotobyname('tcp');
socket(Server, PF_INET, SOCK_STREAM, $proto) || die "Can't create socket: $!";
setsockopt(Server, SOL_SOCKET, SO_REUSEADDR, 1);

# Just in case of problems, let's not hang NetSaint
$SIG{'ALRM'} = sub {
     close(Server);
     select(STDOUT);
     print "No Answer from Client\n";
     exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);

# send ourselves a SIGALRM if we cannot connect to the client
unless (connect(Server, $paddr)) {
     close(Server);
     select(STDOUT);
     print "netsaint_statd inactive on Client\n";
     exit $ERRORS{"CRITICAL"};
}

my $state = "OK";
my $answer = undef;


select(Server);
$| = 1;
print Server "version\n";
my ($servanswer) = <Server>;
alarm(0);
close(Server);
select(STDOUT);

if ($servanswer =~ /Unknown command/i) {
   $state = "WARNING";
   $answer = "statd ver < 2.11; plugin ver $VERSION\n";
} elsif ($servanswer eq $VERSION) {
   $state = "OK";
   $answer = "statd ver == plugin ver == $VERSION\n";
} else {
   $state = "WARNING";
   $answer = "statd ver = $servanswer; plugin ver = $VERSION\n";  
}

print $answer;
exit $ERRORS{$state};

sub usage {
	print "Minimum arguments not supplied!\n";
	print "\n";
	print "Perl Check netsaint_statd plugin for NetSaint\n";
	print "Copyright (c) 2001 tony mancill\n";
	print "\n";
	print "Usage: $0 <host> [<port>]\n";
	print "\n";
	print "<port> = Port that the status daemon is running on <host>.\n	Defaults to 1040.\n";
	exit $ERRORS{"UNKNOWN"};
}
